
/*
Phidgets Inc. 2010
Programmed by: Robert Bruce

The purpose of this program is to open a com-port on a linux machine (Eg: A Phidget SBC), take
any input sent to it, shift the characters by one, and send it back.
*/
#include <stdio.h>
#include "phidget21.h"
#include <string.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>

#define BAUDRATE B38400

#define _POSIX_SOURCE 1
#define FALSE 0
#define TRUE 0

int getbaud(int fd);
int initport(int fd);
int i;

int main()
{
	//Open the Serial port as Read/Write, non controlling terminal, non blocking.
	int file;
	file = open("/dev/ttyUSB0", O_RDWR | O_NOCTTY | O_NONBLOCK);
	if (file < 0)
	{
		printf("Error opening serial port.\n");
	}	
	else
	{
		//Set up the default port settings
		initport(file);
		printf("Serial port connected at %d.\n", getbaud(file));
		while(1)
		{
			char buffer[10];
			//initialize the buffer
			for (i = 0 ; i < sizeof(buffer) ; i++) buffer[i] = '\0';
			
			//read data in from the serial port. The port is set up to accept data line per line,
			//in the initport function, so nothing is returned until the user hits return on the
			//host terminal.
			int result = read(file, buffer,sizeof(buffer));
			if (result > 0) //if we've read in something useful
			{			
				//using strlen() instead of sizeof() to return the size of the useful string, not the whole buffer
				for (i = 0 ; i < strlen(buffer) -1 ; i++)
				{
					//shift data by one. A->B, 1->2, etc.
					if (buffer[i] != '\0') buffer[i]++; 
				}
				//send the data back out to the host terminal.
				write(file, buffer, strlen(buffer));
			}
		}		
	}
	close(file);
	return 0;
}//main

int initport(int fd) {
	struct termios options;
	// Get the current options for the port...
	tcgetattr(fd, &options);

	cfsetispeed(&options, BAUDRATE);
	cfsetospeed(&options, BAUDRATE);

		//Canonical mode, echo characters back to host machine, echo new lines back to host machine
	options.c_lflag |= ICANON | ECHO | ECHONL;
	
	// Enable the receiver and set local mode...
	options.c_cflag |= (CLOCAL | CREAD);
	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;

	// Set the new options for the port...
	tcsetattr(fd, TCSANOW, &options);
	return 1;
}

int getbaud(int fd) {
	struct termios termAttr;
	int inputSpeed = -1;
	speed_t baudRate;
	tcgetattr(fd, &termAttr);
	/* Get the input speed.                              */
	baudRate = cfgetispeed(&termAttr);
	switch (baudRate) {
		case B0:      inputSpeed = 0; break;
		case B50:     inputSpeed = 50; break;
		case B110:    inputSpeed = 110; break;
		case B134:    inputSpeed = 134; break;
		case B150:    inputSpeed = 150; break;
		case B200:    inputSpeed = 200; break;
		case B300:    inputSpeed = 300; break;
		case B600:    inputSpeed = 600; break;
		case B1200:   inputSpeed = 1200; break;
		case B1800:   inputSpeed = 1800; break;
		case B2400:   inputSpeed = 2400; break;
		case B4800:   inputSpeed = 4800; break;
		case B9600:   inputSpeed = 9600; break;
		case B19200:  inputSpeed = 19200; break;
		case B38400:  inputSpeed = 38400; break;
	}
	return inputSpeed;
}
